<?php
/**
 * Comment.
 * Note: Database must use 'utf8mb4' to allow emojis. 'utf8' is not enough.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;

class CommentBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Comments', 'flat101-security' ),
			'comments',
			array(
				'disabled'  => false,
				'HTML'      => true,
				'authority' => true,
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(

			Field::make( 'checkbox', $this->get_option_name( 'disabled' ), __( 'Disabled', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['disabled'] )
			     ->set_help_text(
				     __( 'Used to disable comments completely.', 'flat101-security' )
			     ),

			Field::make( 'checkbox', $this->get_option_name( 'html' ), __( 'HTML', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['HTML'] )
			     ->set_help_text(
				     __( 'Prevent the use of HTML tags in comments, to avoid spam and security vulnerabilities.', 'flat101-security' )
			     )
			     ->set_conditional_logic(
				     array(
					     array(
						     'field' => $this->get_option_name( 'disabled' ),
						     'value' => false,
					     ),
				     )
			     ),

			Field::make( 'checkbox', $this->get_option_name( 'authority' ), __( 'Authority', 'flat101-security' ) )
			     ->set_default_value( $this->default_settings['authority'] )
			     ->set_help_text(
				     __( 'Avoid impersonation of Google authorship in comments.', 'flat101-security' )
			     )
			     ->set_conditional_logic(
				     array(
					     array(
						     'field' => $this->get_option_name( 'disabled' ),
						     'value' => false,
					     ),
				     )
			     ),

		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_description() {
		/* translators: %s: comments options */
		return sprintf( esc_html__( 'Options to control comments.', 'flat101-security' ) );
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		if ( $this->settings['disabled'] ) {

			//Close comments on the front-end
			add_filter( 'comments_open', '__return_false', 20, 2 );
			add_filter( 'pings_open', '__return_false', 20, 2 );

			// Hide existing comments
			add_filter( 'comments_array', '__return_empty_array', 10, 2 );

			// Remove comments
			add_action( 'admin_init', array( $this, 'disabled_comments' ) );

			// Remove comments page in menu
			add_action( 'admin_menu', function () {
				remove_menu_page( 'edit-comments.php' );
			} );

			// Remove comments links from admin bar
			$this->admin_bar();

		} elseif ( $this->settings['HTML'] ) {
			add_filter( 'preprocess_comment', array( $this, 'plc_comment_post' ), '', 1 );
			add_filter( 'comment_text', array( $this, 'plc_comment_display' ), '', 1 );
			add_filter( 'comment_text_rss', array( $this, 'plc_comment_display' ), '', 1 );
			add_filter( 'comment_excerpt', array( $this, 'plc_comment_display' ), '', 1 );

		} elseif ( $this->settings['authority'] ) {
			add_filter( 'preprocess_comment', array( $this, 'dcga_filter_gauthorship' ) );

		}
	}


	/**
	 * Replace comments posts
	 */
	public function plc_comment_post( $incoming_comment ) {
		$incoming_comment['comment_content'] = htmlspecialchars( $incoming_comment['comment_content'] );
		$incoming_comment['comment_content'] = str_replace( "'", '&amp;apos;', $incoming_comment['comment_content'] );

		return ( $incoming_comment );
	}


	/**
	 * Replace comments display
	 */
	public function plc_comment_display( $comment_to_display ) {
		$comment_to_display = str_replace( '&amp;apos;', "'", $comment_to_display );

		return $comment_to_display;
	}


	/**
	 * Disable Comments
	 */
	public function disabled_comments() {
		// Redirect any user trying to access comments page
		global $pagenow;

		if ( 'edit-comments.php' === $pagenow ) {
			wp_redirect( admin_url() );
			exit;
		}

		// Remove comments metabox from dashboard
		remove_meta_box( 'dashboard_recent_comments', 'dashboard', 'normal' );

		// Disable support for comments and trackbacks in post types
		foreach ( get_post_types() as $post_type ) {
			if ( post_type_supports( $post_type, 'comments' ) ) {
				remove_post_type_support( $post_type, 'comments' );
				remove_post_type_support( $post_type, 'trackbacks' );
			}
		}
	}

	/**
	 * Remove comments links from admin bar
	 */
	public function admin_bar() {
		if ( is_admin_bar_showing() ) {
			remove_action( 'admin_bar_menu', 'wp_admin_bar_comments_menu', 60 );
		}
	}

	/**
	 * Avoid impersonation of authority
	 */
	public function dcga_filter_gauthorship( $commentdata ) {
		$pattern = '/http[s]?:\/\/plus.google.com\.*/';
		if ( preg_match( $pattern, $commentdata['comment_content'] ) ) {
			$commentdata['comment_content'] = '';
		}

		return $commentdata;
	}


}
